package net.udby.datetime.ejb;

import static org.junit.Assert.*;

import java.lang.reflect.Proxy;
import java.sql.Timestamp;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

import javax.persistence.EntityManager;

import net.udby.datetime.DateTimeProviderTestImpl;
import net.udby.datetime.derby.DbTestHelper;
import net.udby.datetime.jpa.Item;

import org.junit.After;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

public class ItemServiceEjbTest {

	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		DbTestHelper.initialize();
	}

	private EntityManager em;
	private DateTimeProviderTestImpl dateTimeProvider;
	private ItemService ejbProxy;
	
	@Before
	public void setUp() throws Exception {
		DbTestHelper.truncateAll();
		// Setup the test-data for this unit-test...
		DbTestHelper.runScript("/sql/insert_testdata.sql", true, true);
		em = DbTestHelper.getEntityManagerFactory().createEntityManager();
		dateTimeProvider = new DateTimeProviderTestImpl(0);
		// emulate ejb functionality
		ItemServiceEjb ejb = new ItemServiceEjb();
		ejb.setDateTimeProvider(dateTimeProvider);
		ejb.setEm(em);
		// create service proxy...
		ejbProxy = (ItemService) Proxy.newProxyInstance(this.getClass().getClassLoader(), new Class<?>[]{ ItemService.class }, new EMProxyHandler(em, ejb));
	}

	@After
	public void tearDown() throws Exception {
		em.close();
	}
	
	private static void sort(List<Item> items) {
		Collections.sort(items, new Comparator<Item>() {
			@Override
			public int compare(Item o1, Item o2) {
				return Integer.compare(o1.getId(), o2.getId());
			}
		});
	}

	@Test
	public void testFindActiveItems_20121231() {
		// set "current time" to 2012-12-31 00:00:00.0
		dateTimeProvider.set(Timestamp.valueOf("2012-12-31 00:00:00.0").getTime());
		
		List<Item> items = ejbProxy.findActiveItems();
		
		sort(items);
		
		assertNotNull(items);
		assertEquals(2, items.size());
		assertEquals("03 Active until Jan 2nd", items.get(0).getItemText());
		assertEquals("06 Allways active", items.get(1).getItemText());
	}

	@Test
	public void testFindActiveItems_20130101_235959999() {
		// set "current time" to 2013-01-01 23:59:59.999
		dateTimeProvider.set(Timestamp.valueOf("2013-01-01 23:59:59.999").getTime());
		
		List<Item> items = ejbProxy.findActiveItems();
		
		sort(items);
		
		assertNotNull(items);
		assertEquals(3, items.size());
		assertEquals("03 Active until Jan 2nd", items.get(0).getItemText());
		assertEquals("04 Active Jan 1st..3rd 2013", items.get(1).getItemText());
		assertEquals("06 Allways active", items.get(2).getItemText());
	}

	@Test
	public void testFindActiveItems_20130102() {
		// set "current time" to 2013-01-02 00:00:00.0
		dateTimeProvider.set(Timestamp.valueOf("2013-01-02 00:00:00.0").getTime());
		
		List<Item> items = ejbProxy.findActiveItems();
		
		sort(items);
		
		assertNotNull(items);
		assertEquals(4, items.size());
		assertEquals("02 Active since Jan 2nd", items.get(0).getItemText());
		assertEquals("04 Active Jan 1st..3rd 2013", items.get(1).getItemText());
		assertEquals("05 Active Jan 2nd 2013", items.get(2).getItemText());
		assertEquals("06 Allways active", items.get(3).getItemText());
	}

	@Test
	public void testFindActiveItems_20130102_235959999() {
		// set "current time" to 2013-01-02 23:59:59.999
		dateTimeProvider.set(Timestamp.valueOf("2013-01-02 23:59:59.999").getTime());
		
		List<Item> items = ejbProxy.findActiveItems();
		
		sort(items);
		
		assertNotNull(items);
		assertEquals(4, items.size());
		assertEquals("02 Active since Jan 2nd", items.get(0).getItemText());
		assertEquals("04 Active Jan 1st..3rd 2013", items.get(1).getItemText());
		assertEquals("05 Active Jan 2nd 2013", items.get(2).getItemText());
		assertEquals("06 Allways active", items.get(3).getItemText());
	}

	@Test
	public void testFindActiveItems_20130103() {
		// set "current time" to 2013-01-03 00:00:00.0
		dateTimeProvider.set(Timestamp.valueOf("2013-01-03 00:00:00.0").getTime());
		
		List<Item> items = ejbProxy.findActiveItems();
		
		sort(items);
		
		assertNotNull(items);
		assertEquals(3, items.size());
		assertEquals("02 Active since Jan 2nd", items.get(0).getItemText());
		assertEquals("04 Active Jan 1st..3rd 2013", items.get(1).getItemText());
		assertEquals("06 Allways active", items.get(2).getItemText());
	}

	@Test
	public void testFindActiveItems_20130103_235959999() {
		// set "current time" to 2013-01-03 23:59:59.999
		dateTimeProvider.set(Timestamp.valueOf("2013-01-03 23:59:59.999").getTime());
		
		List<Item> items = ejbProxy.findActiveItems();
		
		sort(items);
		
		assertNotNull(items);
		assertEquals(3, items.size());
		assertEquals("02 Active since Jan 2nd", items.get(0).getItemText());
		assertEquals("04 Active Jan 1st..3rd 2013", items.get(1).getItemText());
		assertEquals("06 Allways active", items.get(2).getItemText());
	}

	@Test
	public void testFindActiveItems_20130104() {
		// set "current time" to 2013-01-04 00:00:00.0
		dateTimeProvider.set(Timestamp.valueOf("2013-01-04 00:00:00.0").getTime());
		
		List<Item> items = ejbProxy.findActiveItems();
		
		sort(items);
		
		assertNotNull(items);
		assertEquals(2, items.size());
		assertEquals("02 Active since Jan 2nd", items.get(0).getItemText());
		assertEquals("06 Allways active", items.get(1).getItemText());
	}
	
	@Test
	public void testCreateItem() {
		Item item = new Item();
		item.setItemText("99 create Test");
		
		item = ejbProxy.create(item);
		
		assertEquals(dateTimeProvider.currentTimeCalendar(), item.getCreated());
		
		Map<String, Object> row = DbTestHelper.selectRow(Item.TABLE_NAME, "ID = ?", true, item.getId());
		
		Timestamp ts = new Timestamp(dateTimeProvider.currentTimeMilliseconds());
		assertEquals(ts, row.get("CREATED"));
		assertEquals("99 create Test", row.get("ITEM_TEXT"));
	}
	
	@Test(expected=IllegalStateException.class)
	public void testGetStatus_minus() {
		Item item = new Item();
		item.setItemText("99 getStatus Test -1");
		
		item = ejbProxy.create(item);
		
		assertEquals(dateTimeProvider.currentTimeCalendar(), item.getCreated());
		
		dateTimeProvider.add(-1);
		// should throw ISE
		ejbProxy.getStatus(item);
	}

	@Test
	public void testGetStatus_0() {
		Item item = new Item();
		item.setItemText("90 getStatus Test 0");
		
		item = ejbProxy.create(item);
		
		assertEquals(dateTimeProvider.currentTimeCalendar(), item.getCreated());		
		assertEquals(Status.GREEN, ejbProxy.getStatus(item));
	}
	
	@Test
	public void testGetStatus_4999() {
		Item item = new Item();
		item.setItemText("91 getStatus Test 4999");
		
		item = ejbProxy.create(item);
		
		assertEquals(dateTimeProvider.currentTimeCalendar(), item.getCreated());
		
		dateTimeProvider.add(4999L);
		
		assertEquals(Status.GREEN, ejbProxy.getStatus(item));
	}
	
	@Test
	public void testGetStatus_5000() {
		Item item = new Item();
		item.setItemText("92 getStatus Test 5000");
		
		item = ejbProxy.create(item);
		
		assertEquals(dateTimeProvider.currentTimeCalendar(), item.getCreated());
		
		dateTimeProvider.add(5000L);

		assertEquals(Status.YELLOW, ejbProxy.getStatus(item));
	}
	
	@Test
	public void testGetStatus_29999() {
		Item item = new Item();
		item.setItemText("93 getStatus Test 29999");
		
		item = ejbProxy.create(item);
		
		assertEquals(dateTimeProvider.currentTimeCalendar(), item.getCreated());		
		
		dateTimeProvider.add(29999L);

		assertEquals(Status.YELLOW, ejbProxy.getStatus(item));
	}
	
	@Test
	public void testGetStatus_30000() {
		Item item = new Item();
		item.setItemText("94 getStatus Test 30000");
		
		item = ejbProxy.create(item);
		
		assertEquals(dateTimeProvider.currentTimeCalendar(), item.getCreated());		
		
		dateTimeProvider.add(30000L);

		assertEquals(Status.RED, ejbProxy.getStatus(item));
	}
}